/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "Heavy_WSTD_SMTHR.hpp"

#include <new>

#define Context(_c) static_cast<Heavy_WSTD_SMTHR *>(_c)


/*
 * C Functions
 */

extern "C" {
  HV_EXPORT HeavyContextInterface *hv_WSTD_SMTHR_new(double sampleRate) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_SMTHR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_SMTHR(sampleRate);
    return Context(ptr);
  }

  HV_EXPORT HeavyContextInterface *hv_WSTD_SMTHR_new_with_options(double sampleRate,
      int poolKb, int inQueueKb, int outQueueKb) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_SMTHR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_SMTHR(sampleRate, poolKb, inQueueKb, outQueueKb);
    return Context(ptr);
  }

  HV_EXPORT void hv_WSTD_SMTHR_free(HeavyContextInterface *instance) {
    // call destructor
    Context(instance)->~Heavy_WSTD_SMTHR();
    // free memory
    hv_free(instance);
  }
} // extern "C"







/*
 * Class Functions
 */

Heavy_WSTD_SMTHR::Heavy_WSTD_SMTHR(double sampleRate, int poolKb, int inQueueKb, int outQueueKb)
    : HeavyContext(sampleRate, poolKb, inQueueKb, outQueueKb) {
  numBytes += sLine_init(&sLine_9TiDDxEh);
  numBytes += sEnv_init(&sEnv_c1SBAloV, 512, 256);
  numBytes += sEnv_init(&sEnv_8tjTSLrA, 512, 256);
  numBytes += sEnv_init(&sEnv_nmhHe9rD, 512, 256);
  numBytes += sEnv_init(&sEnv_nXI3tEKj, 512, 256);
  numBytes += sLine_init(&sLine_IlF75Dna);
  numBytes += sLine_init(&sLine_Zjemn9CK);
  numBytes += cVar_init_f(&cVar_diZ1uLJk, 0.0f);
  numBytes += cVar_init_f(&cVar_cVFLC3Eq, 0.0f);
  numBytes += sVarf_init(&sVarf_tMaIIPhP, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_Li3sgRyG, 0.0f);
  numBytes += cIf_init(&cIf_nRoIY4ei, false);
  numBytes += cIf_init(&cIf_zl8X9Rjz, false);
  numBytes += cBinop_init(&cBinop_kVuHQNhh, 0.0f); // __sub
  numBytes += sVarf_init(&sVarf_ZXu77Und, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_gXs2gGb8, 0.0f);
  numBytes += cIf_init(&cIf_cOnHSXN6, false);
  numBytes += cIf_init(&cIf_gMKEUPCh, false);
  numBytes += cBinop_init(&cBinop_GDiJ5JhW, 0.0f); // __sub
  numBytes += cVar_init_f(&cVar_8RPPt3DA, 1.0f);
  numBytes += sVarf_init(&sVarf_eVVsCTL8, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_2s5JgflV, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_sNgCN0dH, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_y37MecFv, 0.0f, 0.0f, false);
  
  // schedule a message to trigger all loadbangs via the __hv_init receiver
  scheduleMessageForReceiver(0xCE5CC65B, msg_initWithBang(HV_MESSAGE_ON_STACK(1), 0));
}

Heavy_WSTD_SMTHR::~Heavy_WSTD_SMTHR() {
  sEnv_free(&sEnv_c1SBAloV);
  sEnv_free(&sEnv_8tjTSLrA);
  sEnv_free(&sEnv_nmhHe9rD);
  sEnv_free(&sEnv_nXI3tEKj);
}

HvTable *Heavy_WSTD_SMTHR::getTableForHash(hv_uint32_t tableHash) {
  return nullptr;
}

void Heavy_WSTD_SMTHR::scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) {
  switch (receiverHash) {
    case 0xCB23DC60: { // Lmtr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_6xR0QvKR_sendMessage);
      break;
    }
    case 0x97D46AC6: { // Mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_sZ7dmOQ5_sendMessage);
      break;
    }
    case 0xD88E26F9: { // Smthr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_JkN8SpFn_sendMessage);
      break;
    }
    case 0xCF8296B9: { // 1001-mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_JOTw5YeA_sendMessage);
      break;
    }
    case 0xCE5CC65B: { // __hv_init
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_I9vQkAAc_sendMessage);
      break;
    }
    default: return;
  }
}

int Heavy_WSTD_SMTHR::getParameterInfo(int index, HvParameterInfo *info) {
  if (info != nullptr) {
    switch (index) {
      case 0: {
        info->name = "Lmtr";
        info->hash = 0xCB23DC60;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 1.0f;
        info->defaultVal = 1.0f;
        break;
      }
      case 1: {
        info->name = "Mix";
        info->hash = 0x97D46AC6;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 100.0f;
        info->defaultVal = 50.0f;
        break;
      }
      case 2: {
        info->name = "Smthr";
        info->hash = 0xD88E26F9;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 1.0f;
        info->maxVal = 13.37f;
        info->defaultVal = 1.0f;
        break;
      }
      default: {
        info->name = "invalid parameter index";
        info->hash = 0;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 0.0f;
        info->defaultVal = 0.0f;
        break;
      }
    }
  }
  return 3;
}



/*
 * Send Function Implementations
 */


void Heavy_WSTD_SMTHR::cBinop_iO1UoYXv_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MAX, 0.0f, 0, m, &cBinop_Fi8qkfa1_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_Fi8qkfa1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 2.0f, 0, m, &cBinop_1bSuO8zO_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_1bSuO8zO_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 1.0f, 0, m, &cBinop_wLbUGIR1_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_wLbUGIR1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_ZFXGR8kl_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_AdoGPX4D_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_ZFXGR8kl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_ADD, 1.0f, 0, m, &cBinop_Qd21TUJp_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_AdoGPX4D_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_MOZ2LveK_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cBinop_Qd21TUJp_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_MG3oEayf_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_MOZ2LveK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_85jNr1gf_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_85jNr1gf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_WOr3XVuO_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_MG3oEayf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_Bo1kHzTb_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cBinop_WOr3XVuO_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_2vfCYAst_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cVar_diZ1uLJk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cSend_iocHJIff_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cCast_wxoGIou8_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_Fp9CQKc4_sendMessage(_c, 0, m);
  cMsg_5cQO0TpY_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cVar_cVFLC3Eq_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_ftQ5Fcc2_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cVar_Li3sgRyG_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_kVuHQNhh, HV_BINOP_SUBTRACT, 0, m, &cBinop_kVuHQNhh_sendMessage);
}

void Heavy_WSTD_SMTHR::cIf_nRoIY4ei_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_tMaIIPhP, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::cIf_zl8X9Rjz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_tMaIIPhP, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::sEnv_c1SBAloV_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_Li3sgRyG, 0, m, &cVar_Li3sgRyG_sendMessage);
}

void Heavy_WSTD_SMTHR::sEnv_8tjTSLrA_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_j4mVCN3I_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_0GeLY7vt_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_iHKKpagN_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_kVuHQNhh_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_7TK8ZgwN_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cMsg_7TK8ZgwN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_OD7ak0mH_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_OD7ak0mH_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_uWm4Qk3w_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_eG85bxiR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_oNxq8Cau_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_TIgVUPY4_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_TIgVUPY4_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_tYHrK6dM_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cCast_oNxq8Cau_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_nRoIY4ei, 1, m, &cIf_nRoIY4ei_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_2yyKmqiw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_zl8X9Rjz, 1, m, &cIf_zl8X9Rjz_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_tYHrK6dM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_nRoIY4ei, 0, m, &cIf_nRoIY4ei_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_uWm4Qk3w_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_zl8X9Rjz, 0, m, &cIf_zl8X9Rjz_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_0GeLY7vt_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_kVuHQNhh, HV_BINOP_SUBTRACT, 1, m, &cBinop_kVuHQNhh_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_j4mVCN3I_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_SMTHR::cCast_iHKKpagN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_Li3sgRyG, 0, m, &cVar_Li3sgRyG_sendMessage);
}

void Heavy_WSTD_SMTHR::cVar_gXs2gGb8_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_GDiJ5JhW, HV_BINOP_SUBTRACT, 0, m, &cBinop_GDiJ5JhW_sendMessage);
}

void Heavy_WSTD_SMTHR::cIf_cOnHSXN6_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_ZXu77Und, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::cIf_gMKEUPCh_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_ZXu77Und, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::sEnv_nmhHe9rD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_gXs2gGb8, 0, m, &cVar_gXs2gGb8_sendMessage);
}

void Heavy_WSTD_SMTHR::sEnv_nXI3tEKj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_jERttSM5_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_NnAuJhEM_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_ADLDix2s_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_GDiJ5JhW_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_oZjsC9u5_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cMsg_oZjsC9u5_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_YPQAiJZP_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_YPQAiJZP_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_0zbJpR9i_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_kMkNWzSK_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_MuOk1Rzz_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_kgrm5k7f_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_MuOk1Rzz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_cOnHSXN6, 1, m, &cIf_cOnHSXN6_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_kgrm5k7f_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_GBV7bVPe_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cBinop_9SxvsBbv_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_gMKEUPCh, 1, m, &cIf_gMKEUPCh_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_GBV7bVPe_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_cOnHSXN6, 0, m, &cIf_cOnHSXN6_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_0zbJpR9i_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_gMKEUPCh, 0, m, &cIf_gMKEUPCh_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_jERttSM5_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_SMTHR::cCast_ADLDix2s_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_gXs2gGb8, 0, m, &cVar_gXs2gGb8_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_NnAuJhEM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_GDiJ5JhW, HV_BINOP_SUBTRACT, 1, m, &cBinop_GDiJ5JhW_sendMessage);
}

void Heavy_WSTD_SMTHR::cVar_8RPPt3DA_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_sQZvWi9C_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_eG85bxiR_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_2yyKmqiw_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_kMkNWzSK_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_9SxvsBbv_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_sQZvWi9C_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_8RPPt3DA, 1, m, &cVar_8RPPt3DA_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_Bo1kHzTb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_Zjemn9CK, 0, m, NULL);
}

void Heavy_WSTD_SMTHR::cMsg_2vfCYAst_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_IlF75Dna, 0, m, NULL);
}

void Heavy_WSTD_SMTHR::cSend_iocHJIff_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cReceive_JOTw5YeA_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cBinop_iM6LLTXG_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_diZ1uLJk, 0, m, &cVar_diZ1uLJk_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_5cQO0TpY_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 50.0f);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_iM6LLTXG_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_ftQ5Fcc2_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 20.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_9TiDDxEh, 0, m, NULL);
}

void Heavy_WSTD_SMTHR::cMsg_Fp9CQKc4_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  cVar_onMessage(_c, &Context(_c)->cVar_cVFLC3Eq, 0, m, &cVar_cVFLC3Eq_sendMessage);
  cVar_onMessage(_c, &Context(_c)->cVar_8RPPt3DA, 0, m, &cVar_8RPPt3DA_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_JOTw5YeA_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 1.0f, 0, m, &cBinop_iO1UoYXv_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_sZ7dmOQ5_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_iM6LLTXG_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_I9vQkAAc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_wxoGIou8_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_6xR0QvKR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_8RPPt3DA, 0, m, &cVar_8RPPt3DA_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_JkN8SpFn_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_cVFLC3Eq, 0, m, &cVar_cVFLC3Eq_sendMessage);
}




/*
 * Context Process Implementation
 */

int Heavy_WSTD_SMTHR::process(float **inputBuffers, float **outputBuffers, int n) {
  while (hLp_hasData(&inQueue)) {
    hv_uint32_t numBytes = 0;
    ReceiverMessagePair *p = reinterpret_cast<ReceiverMessagePair *>(hLp_getReadBuffer(&inQueue, &numBytes));
    hv_assert(numBytes >= sizeof(ReceiverMessagePair));
    scheduleMessageForReceiver(p->receiverHash, &p->msg);
    hLp_consume(&inQueue);
  }

  sendBangToReceiver(0xDD21C0EB); // send to __hv_bang~ on next cycle
  const int n4 = n & ~HV_N_SIMD_MASK; // ensure that the block size is a multiple of HV_N_SIMD

  // temporary signal vars
  hv_bufferf_t Bf0, Bf1, Bf2, Bf3, Bf4, Bf5;

  // input and output vars
  hv_bufferf_t O0, O1;
  hv_bufferf_t I0, I1;

  // declare and init the zero buffer
  hv_bufferf_t ZERO; __hv_zero_f(VOf(ZERO));

  hv_uint32_t nextBlock = blockStartTimestamp;
  for (int n = 0; n < n4; n += HV_N_SIMD) {

    // process all of the messages for this block
    nextBlock += HV_N_SIMD;
    while (mq_hasMessageBefore(&mq, nextBlock)) {
      MessageNode *const node = mq_peek(&mq);
      node->sendMessage(this, node->let, node->m);
      mq_pop(&mq);
    }

    // load input buffers
    __hv_load_f(inputBuffers[0]+n, VOf(I0));
    __hv_load_f(inputBuffers[1]+n, VOf(I1));

    // zero output buffers
    __hv_zero_f(VOf(O0));
    __hv_zero_f(VOf(O1));

    // process all signal functions
    __hv_varwrite_f(&sVarf_eVVsCTL8, VIf(I0));
    __hv_line_f(&sLine_9TiDDxEh, VOf(Bf0));
    __hv_mul_f(VIf(I0), VIf(Bf0), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f);
    __hv_min_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f);
    __hv_max_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf1), VIf(Bf1), VOf(Bf2));
    __hv_var_k_f(VOf(Bf3), 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f);
    __hv_add_f(VIf(Bf2), VIf(Bf3), VOf(Bf4));
    __hv_var_k_f(VOf(Bf5), 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f);
    __hv_fma_f(VIf(Bf2), VIf(Bf5), VIf(Bf3), VOf(Bf3));
    __hv_div_f(VIf(Bf4), VIf(Bf3), VOf(Bf3));
    __hv_mul_f(VIf(Bf1), VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_c1SBAloV, VIf(Bf3), &sEnv_c1SBAloV_sendMessage);
    sEnv_process(this, &sEnv_8tjTSLrA, VIf(I0), &sEnv_8tjTSLrA_sendMessage);
    __hv_varread_f(&sVarf_tMaIIPhP, VOf(Bf1));
    __hv_var_k_f(VOf(Bf4), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf4), VOf(Bf4));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf4), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_2s5JgflV, VIf(Bf1));
    __hv_varwrite_f(&sVarf_sNgCN0dH, VIf(I1));
    __hv_mul_f(VIf(I1), VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f);
    __hv_min_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f);
    __hv_max_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_mul_f(VIf(Bf0), VIf(Bf0), VOf(Bf1));
    __hv_var_k_f(VOf(Bf3), 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f);
    __hv_add_f(VIf(Bf1), VIf(Bf3), VOf(Bf4));
    __hv_var_k_f(VOf(Bf5), 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f);
    __hv_fma_f(VIf(Bf1), VIf(Bf5), VIf(Bf3), VOf(Bf3));
    __hv_div_f(VIf(Bf4), VIf(Bf3), VOf(Bf3));
    __hv_mul_f(VIf(Bf0), VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_nmhHe9rD, VIf(Bf3), &sEnv_nmhHe9rD_sendMessage);
    sEnv_process(this, &sEnv_nXI3tEKj, VIf(I1), &sEnv_nXI3tEKj_sendMessage);
    __hv_varread_f(&sVarf_ZXu77Und, VOf(Bf0));
    __hv_var_k_f(VOf(Bf4), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf0), VIf(Bf4), VOf(Bf4));
    __hv_var_k_f(VOf(Bf0), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf4), VIf(Bf0), VOf(Bf0));
    __hv_exp_f(VIf(Bf0), VOf(Bf0));
    __hv_mul_f(VIf(Bf3), VIf(Bf0), VOf(Bf0));
    __hv_varwrite_f(&sVarf_y37MecFv, VIf(Bf0));
    __hv_varread_f(&sVarf_sNgCN0dH, VOf(Bf0));
    __hv_line_f(&sLine_IlF75Dna, VOf(Bf3));
    __hv_varread_f(&sVarf_y37MecFv, VOf(Bf4));
    __hv_line_f(&sLine_Zjemn9CK, VOf(Bf5));
    __hv_mul_f(VIf(Bf4), VIf(Bf5), VOf(Bf4));
    __hv_fma_f(VIf(Bf0), VIf(Bf3), VIf(Bf4), VOf(Bf4));
    __hv_add_f(VIf(Bf4), VIf(O1), VOf(O1));
    __hv_varread_f(&sVarf_eVVsCTL8, VOf(Bf4));
    __hv_varread_f(&sVarf_2s5JgflV, VOf(Bf0));
    __hv_mul_f(VIf(Bf0), VIf(Bf5), VOf(Bf5));
    __hv_fma_f(VIf(Bf4), VIf(Bf3), VIf(Bf5), VOf(Bf5));
    __hv_add_f(VIf(Bf5), VIf(O0), VOf(O0));

    // save output vars to output buffer
    __hv_store_f(outputBuffers[0]+n, VIf(O0));
    __hv_store_f(outputBuffers[1]+n, VIf(O1));
  }

  blockStartTimestamp = nextBlock;

  return n4; // return the number of frames processed

}

int Heavy_WSTD_SMTHR::processInline(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(!(n4 & HV_N_SIMD_MASK)); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s)
  float **const bIn = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bIn[0] = inputBuffers+(0*n4);
  bIn[1] = inputBuffers+(1*n4);

  // define the heavy output buffer for 2 channel(s)
  float **const bOut = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bOut[0] = outputBuffers+(0*n4);
  bOut[1] = outputBuffers+(1*n4);

  int n = process(bIn, bOut, n4);
  return n;
}

int Heavy_WSTD_SMTHR::processInlineInterleaved(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(n4 & ~HV_N_SIMD_MASK); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s), uninterleave
  float *const bIn = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));
  #if HV_SIMD_SSE || HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 a = _mm_load_ps(inputBuffers+i);                // LRLR
    __m128 b = _mm_load_ps(inputBuffers+4+i);              // LRLR
    __m128 x = _mm_shuffle_ps(a, b, _MM_SHUFFLE(2,0,2,0)); // LLLL
    __m128 y = _mm_shuffle_ps(a, b, _MM_SHUFFLE(3,1,3,1)); // RRRR
    _mm_store_ps(bIn+j, x);
    _mm_store_ps(bIn+n4+j, y);
  }
  #elif HV_SIMD_NEON
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4x2_t a = vld2q_f32(inputBuffers+i); // load and uninterleave
    vst1q_f32(bIn+j, a.val[0]);
    vst1q_f32(bIn+n4+j, a.val[1]);
  }
  #else // HV_SIMD_NONE
  for (int j = 0; j < n4; ++j) {
    bIn[0*n4+j] = inputBuffers[0+2*j];
    bIn[1*n4+j] = inputBuffers[1+2*j];
  }
  #endif

  // define the heavy output buffer for 2 channel(s)
  float *const bOut = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));

  int n = processInline(bIn, bOut, n4);

  // interleave the heavy output into the output buffer
  #if HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 8, i += 16) {
    __m256 x = _mm256_load_ps(bOut+j);    // LLLLLLLL
    __m256 y = _mm256_load_ps(bOut+n4+j); // RRRRRRRR
    __m256 a = _mm256_unpacklo_ps(x, y);  // LRLRLRLR
    __m256 b = _mm256_unpackhi_ps(x, y);  // LRLRLRLR
    _mm256_store_ps(outputBuffers+i, a);
    _mm256_store_ps(outputBuffers+8+i, b);
  }
  #elif HV_SIMD_SSE
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 x = _mm_load_ps(bOut+j);    // LLLL
    __m128 y = _mm_load_ps(bOut+n4+j); // RRRR
    __m128 a = _mm_unpacklo_ps(x, y);  // LRLR
    __m128 b = _mm_unpackhi_ps(x, y);  // LRLR
    _mm_store_ps(outputBuffers+i, a);
    _mm_store_ps(outputBuffers+4+i, b);
  }
  #elif HV_SIMD_NEON
  // https://community.arm.com/groups/processors/blog/2012/03/13/coding-for-neon--part-5-rearranging-vectors
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4_t x = vld1q_f32(bOut+j);
    float32x4_t y = vld1q_f32(bOut+n4+j);
    float32x4x2_t z = {x, y};
    vst2q_f32(outputBuffers+i, z); // interleave and store
  }
  #else // HV_SIMD_NONE
  for (int i = 0; i < 2; ++i) {
    for (int j = 0; j < n4; ++j) {
      outputBuffers[i+2*j] = bOut[i*n4+j];
    }
  }
  #endif

  return n;
}
