/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvControlIf.h"
#include "HvMath.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvSignalEnvelope.h"
#include "HvControlBinop.h"
#include "HvSignalVar.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_iO1UoYXv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Fi8qkfa1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1bSuO8zO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wLbUGIR1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ZFXGR8kl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_AdoGPX4D_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Qd21TUJp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_MOZ2LveK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_85jNr1gf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MG3oEayf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WOr3XVuO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_diZ1uLJk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_wxoGIou8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_cVFLC3Eq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Li3sgRyG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_nRoIY4ei_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_zl8X9Rjz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_c1SBAloV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_8tjTSLrA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kVuHQNhh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_7TK8ZgwN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OD7ak0mH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_eG85bxiR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_TIgVUPY4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oNxq8Cau_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_2yyKmqiw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_tYHrK6dM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_uWm4Qk3w_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0GeLY7vt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_j4mVCN3I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_iHKKpagN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_gXs2gGb8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_cOnHSXN6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_gMKEUPCh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_nmhHe9rD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_nXI3tEKj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GDiJ5JhW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_oZjsC9u5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YPQAiJZP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_kMkNWzSK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_MuOk1Rzz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kgrm5k7f_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_9SxvsBbv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_GBV7bVPe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0zbJpR9i_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_jERttSM5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ADLDix2s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_NnAuJhEM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_8RPPt3DA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_sQZvWi9C_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Bo1kHzTb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_2vfCYAst_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_iocHJIff_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iM6LLTXG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_5cQO0TpY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_ftQ5Fcc2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Fp9CQKc4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_JOTw5YeA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_sZ7dmOQ5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_I9vQkAAc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_6xR0QvKR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_JkN8SpFn_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_9TiDDxEh;
  SignalEnvelope sEnv_c1SBAloV;
  SignalEnvelope sEnv_8tjTSLrA;
  SignalEnvelope sEnv_nmhHe9rD;
  SignalEnvelope sEnv_nXI3tEKj;
  SignalLine sLine_IlF75Dna;
  SignalLine sLine_Zjemn9CK;
  ControlBinop cBinop_iO1UoYXv;
  ControlBinop cBinop_Fi8qkfa1;
  ControlBinop cBinop_1bSuO8zO;
  ControlBinop cBinop_wLbUGIR1;
  ControlBinop cBinop_Qd21TUJp;
  ControlBinop cBinop_85jNr1gf;
  ControlBinop cBinop_MG3oEayf;
  ControlBinop cBinop_WOr3XVuO;
  ControlVar cVar_diZ1uLJk;
  ControlVar cVar_cVFLC3Eq;
  SignalVarf sVarf_tMaIIPhP;
  ControlVar cVar_Li3sgRyG;
  ControlIf cIf_nRoIY4ei;
  ControlIf cIf_zl8X9Rjz;
  ControlBinop cBinop_kVuHQNhh;
  ControlBinop cBinop_OD7ak0mH;
  ControlBinop cBinop_eG85bxiR;
  ControlBinop cBinop_2yyKmqiw;
  ControlBinop cBinop_uWm4Qk3w;
  SignalVarf sVarf_ZXu77Und;
  ControlVar cVar_gXs2gGb8;
  ControlIf cIf_cOnHSXN6;
  ControlIf cIf_gMKEUPCh;
  ControlBinop cBinop_GDiJ5JhW;
  ControlBinop cBinop_YPQAiJZP;
  ControlBinop cBinop_kMkNWzSK;
  ControlBinop cBinop_9SxvsBbv;
  ControlBinop cBinop_0zbJpR9i;
  ControlVar cVar_8RPPt3DA;
  ControlBinop cBinop_sQZvWi9C;
  SignalVarf sVarf_eVVsCTL8;
  SignalVarf sVarf_2s5JgflV;
  SignalVarf sVarf_sNgCN0dH;
  SignalVarf sVarf_y37MecFv;
  ControlBinop cBinop_iM6LLTXG;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
